/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8240_SUPSTP_PRECO_ECON_AF                                            */
/*                                                                            */
/* Descrio:                                                                 */
/* Calcula o preo econmico para um determinado produto da AF                */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 09/07/99 - Yoris L. de Souza                                               */
/*                                                                            */
/* Manutenes:                                                               */
/* 29/07/1999 - Alteracao na forma de calculo do Preco Economico para Forma de*/
/* pagamento Variavel e arredondamento do resultado dos calculos de Preco Eco.*/
/* 12/01/2000 - Troca para STP                                                */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* PARAMETROS:                                                                */
/* P_CDFILIAL    - Codigo da Central de Compras.                              */
/* P_CDFILDEST   - Codigo da Filial Destino                                   */
/* P_CDFILLANC   - Codigo da Filial Lancamento                                */
/* P_NRAUTOFORN  - Numero da AF                                               */
/* P_CDPRODUTO   - Codigo do Produto                                          */
/* P_CDMARCA     - Codigo da Marca                                            */
/* P_DTPROGENTAF - Data de entrega do produto                                 */
/******************************************************************************/
/* @D:\SQL\PRECO_ECO_AF\8240_SUPSTP_PRECO_ECON_AF.SQL                         */
/* @H:\SISTEMAS\DBA\SQL\ORACLE\8240_SUPSTP_PRECO_ECON_AF.SQL                  */
/******************************************************************************/


CREATE OR REPLACE PROCEDURE PRECO_ECON_AF
  (P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_CDFILDEST    IN AUTOFORNE.CDFILDEST%TYPE,
   P_CDFILLANC    IN AUTOFORNE.CDFILLANC%TYPE,
   P_NRAUTOFORN   IN AUTOFORNE.NRAUTOFORN%TYPE,
   P_CDPRODUTO    IN ITEMAUTF.CDPRODUTO%TYPE,
   P_CDMARCA      IN ITEMAUTF.CDMARCA%TYPE,
   P_DTPROGENTAF  IN ITEMAUTF.DTPROGENTAF%TYPE,
   P_VRPRECECO    OUT NUMBER) 
IS

   V_DTPARCELA    DATE;
   V_NRDIAS       NUMBER;
   V_NRFATOR      NUMBER;
   V_VRINDICE     NUMBER;
   V_DTULDIAMES   DATE;
   V_DTULTAUX     DATE;
   V_FATORPARC    NUMBER;
   V_VRPRECECO    NUMBER;
   V_VRPZOPGT     NUMBER;
   V_CONTA        NUMBER;
   V_DTPRIPARC    DATE;
   V_TOTREST      NUMBER;

   V_DTAUTOFORN   AUTOFORNE.DTAUTOFORN%TYPE;
   V_CDFORMPGTO   AUTOFORNE.CDFORMPGTO%TYPE;
   V_IDTPFORPGTAF AUTOFORNE.IDTPFORPGTAF%TYPE;
   V_NRPARCPGTOAF AUTOFORNE.NRPARCPGTOAF%TYPE;
   V_NRDIAPGFIXAF AUTOFORNE.NRDIAPGFIXAF%TYPE;
   V_IDUTDIAPGTAF AUTOFORNE.IDUTDIAPGTAF%TYPE;
   V_VRPREUNIAF   ITEMAUTF.VRPRECECOAF%TYPE;
   V_VRPERDESAF   ITEMAUTF.VRPERDESAF%TYPE;

   V_VRIMPITM     IMPITMAF.VRADIMPAF%TYPE;
   V_VRRENDATUMAR MARCPROD.VRRENDATUMAR%TYPE;

 CURSOR C_PARCELAS IS
  SELECT
   V_DTPRIPARC +
   PRAZOPGTAF.NRDIASPGTOAF AS DTPARCELA,
   PRAZOPGTAF.VRPERCPGTOAF AS VRPZOPGT
  FROM
   PRAZOPGTAF
  WHERE
   PRAZOPGTAF.CDFILDEST     = P_CDFILDEST AND
   PRAZOPGTAF.CDFILLANC     = P_CDFILLANC AND
   PRAZOPGTAF.NRAUTOFORN    = P_NRAUTOFORN;

 CURSOR C_PARCFIXAS IS
       SELECT
        TO_DATE(LPAD(TO_CHAR(NRDIAPGFIXAF),2,'0') ||
        TO_CHAR(DTAUTOFORN,'MMYYYY'),'DDMMYYYY')
       FROM
         AUTOFORNE
       WHERE
        AUTOFORNE.CDFILDEST      = P_CDFILDEST AND
        AUTOFORNE.CDFILLANC      = P_CDFILLANC AND
        AUTOFORNE.NRAUTOFORN     = P_NRAUTOFORN AND
        TO_DATE(LPAD(TO_CHAR(NRDIAPGFIXAF),2,'0') ||
        TO_CHAR(DTAUTOFORN,'MMYYYY'),'DDMMYYYY') >
        DTAUTOFORN AND
        IDUTDIAPGTAF = 'I'
       ---------------
       UNION
       ---------------
       SELECT
        NRDIAPGFIXAF +
        LAST_DAY(DTAUTOFORN)
       FROM
        AUTOFORNE
       WHERE
        AUTOFORNE.CDFILDEST      = P_CDFILDEST AND
        AUTOFORNE.CDFILLANC      = P_CDFILLANC AND
        AUTOFORNE.NRAUTOFORN     = P_NRAUTOFORN AND
        TO_DATE(LPAD(TO_CHAR(NRDIAPGFIXAF),2,'0') ||
        TO_CHAR(DTAUTOFORN,'MMYYYY'),'DDMMYYYY') <=
        DTAUTOFORN AND
        IDUTDIAPGTAF = 'I'
       ---------------
       UNION
       ---------------
       SELECT
        NRDIAPGFIXAF +
        LAST_DAY(DTAUTOFORN)
       FROM
        AUTOFORNE
       WHERE
        AUTOFORNE.CDFILDEST      = P_CDFILDEST AND
        AUTOFORNE.CDFILLANC      = P_CDFILLANC AND
        AUTOFORNE.NRAUTOFORN     = P_NRAUTOFORN AND
        IDUTDIAPGTAF = 'E';


BEGIN

  SELECT
   AUTOFORNE.CDFORMPGTO,
   AUTOFORNE.IDTPFORPGTAF,
   AUTOFORNE.NRPARCPGTOAF,
   AUTOFORNE.NRDIAPGFIXAF,
   AUTOFORNE.IDUTDIAPGTAF,
   AUTOFORNE.DTAUTOFORN,
   ITEMAUTF.VRPREUNIAF,
   ITEMAUTF.VRPERDESAF
   INTO V_CDFORMPGTO,
        V_IDTPFORPGTAF,
        V_NRPARCPGTOAF,
        V_NRDIAPGFIXAF,
        V_IDUTDIAPGTAF,
        V_DTAUTOFORN,
        V_VRPREUNIAF,
        V_VRPERDESAF
  FROM AUTOFORNE, ITEMAUTF
  WHERE
   AUTOFORNE.CDFILDEST  = P_CDFILDEST AND
   AUTOFORNE.CDFILLANC  = P_CDFILLANC AND
   AUTOFORNE.NRAUTOFORN = P_NRAUTOFORN AND
   ITEMAUTF.CDFILDEST   = AUTOFORNE.CDFILDEST AND
   ITEMAUTF.CDFILLANC   = AUTOFORNE.CDFILLANC AND
   ITEMAUTF.NRAUTOFORN  = AUTOFORNE.NRAUTOFORN AND
   ITEMAUTF.CDPRODUTO   = P_CDPRODUTO AND
   ITEMAUTF.CDMARCA     = P_CDMARCA AND
   ITEMAUTF.DTPROGENTAF = P_DTPROGENTAF;

   SELECT
     MAX(MARCPROD.VRRENDATUMAR) INTO V_VRRENDATUMAR
   FROM MARCPROD
   WHERE
     MARCPROD.CDPRODUTO  =  P_CDPRODUTO AND
     MARCPROD.CDMARCA    =  P_CDMARCA;

   SELECT
     SUM(VRADIMPAF) INTO V_VRIMPITM
   FROM
     IMPITMAF
   WHERE
     CDFILDEST   = P_CDFILDEST   AND
     CDFILLANC   = P_CDFILLANC   AND
     NRAUTOFORN  = P_NRAUTOFORN  AND
     CDPRODUTO   = P_CDPRODUTO   AND
     CDMARCA     = P_CDMARCA     AND
     DTPROGENTAF = DTPROGENTAF;

  IF V_VRPERDESAF IS NULL THEN
     V_VRPERDESAF := 0;
  END IF;
  IF V_VRRENDATUMAR IS NULL THEN
     V_VRRENDATUMAR := 0;
  END IF;
  IF V_VRIMPITM IS NULL THEN
     V_VRIMPITM := 0;
  END IF;
  V_VRPRECECO := 0;

  IF V_IDTPFORPGTAF = 'V' THEN
    SELECT
     MIN(ROUND((TO_NUMBER(NVL(DIAPGTO.NRDIAFINPGTO,1)) - 1)/2) +
     AUTOFORNE.DTAUTOFORN) INTO V_DTPRIPARC
    FROM
     DIAPGTO, AUTOFORNE
    WHERE
     DIAPGTO.CDFORMPGTO               = V_CDFORMPGTO AND
     TO_NUMBER(DIAPGTO.NRDIAINIPGTO ) = 1   AND
     AUTOFORNE.CDFILDEST      = P_CDFILDEST AND
     AUTOFORNE.CDFILLANC      = P_CDFILLANC AND
     AUTOFORNE.NRAUTOFORN     = P_NRAUTOFORN;

    IF V_DTPRIPARC IS NULL THEN
       V_DTPRIPARC := V_DTAUTOFORN;
    END IF;

    V_TOTREST := 0;
    OPEN C_PARCELAS;
    LOOP
       FETCH C_PARCELAS INTO V_DTPARCELA, V_VRPZOPGT;
       EXIT WHEN C_PARCELAS%NOTFOUND;
       V_NRFATOR := 1;

       SELECT
          LAST_DAY(V_DTAUTOFORN),LAST_DAY(V_DTAUTOFORN)-V_DTAUTOFORN
          INTO V_DTULDIAMES, V_NRDIAS
       FROM PARAMGERAL;

       V_DTULTAUX := V_DTAUTOFORN;
       WHILE V_DTPARCELA > V_DTULDIAMES LOOP

         SELECT MAX(MOVINDICE.VRINDICE) INTO V_VRINDICE
         FROM
          PARASUPRI, MOVINDICE
         WHERE
          PARASUPRI.CDFILIAL   = P_CDFILIAL           AND
          TO_CHAR(MOVINDICE.DTREFIND,'MMYYYY') =
          TO_CHAR(V_DTULDIAMES,'MMYYYY')               AND
          PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
          PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
          PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

         IF V_VRINDICE IS NULL THEN
            V_VRINDICE := 0;
         END IF;

         V_NRFATOR := V_NRFATOR * (1 + (V_VRINDICE/100))**(V_NRDIAS/30);

         V_DTULTAUX := V_DTULDIAMES;

         SELECT LAST_DAY(V_DTULDIAMES+1),LAST_DAY(V_DTULDIAMES+1)-V_DTULDIAMES
            INTO V_DTULDIAMES, V_NRDIAS
         FROM PARAMGERAL;

       END LOOP;

       V_NRDIAS := V_DTPARCELA - V_DTULTAUX;

       SELECT MAX(MOVINDICE.VRINDICE) INTO V_VRINDICE
       FROM
        PARASUPRI, MOVINDICE
       WHERE
        PARASUPRI.CDFILIAL   = P_CDFILIAL           AND
        TO_CHAR(MOVINDICE.DTREFIND,'MMYYYY') =
        TO_CHAR(V_DTPARCELA,'MMYYYY')               AND
        PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
        PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
        PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

       IF V_VRINDICE IS NULL THEN
          V_VRINDICE := 0;
       END IF;

       V_NRFATOR := V_NRFATOR * (1 + (V_VRINDICE/100))**(V_NRDIAS/30);

       V_TOTREST := V_TOTREST + ((1 / V_NRFATOR) * (V_VRPZOPGT/100));

    END LOOP;
    CLOSE C_PARCELAS;
    V_VRPRECECO := ROUND((V_VRPREUNIAF - (V_VRPREUNIAF * (V_VRPERDESAF /100)) + V_VRIMPITM) *
                   (1/(1+(V_VRRENDATUMAR/100))) * V_TOTREST,5);


  ELSIF V_IDTPFORPGTAF = 'F' THEN

      OPEN C_PARCFIXAS;
      FETCH C_PARCFIXAS INTO V_DTPARCELA;
      CLOSE C_PARCFIXAS;
      
      V_FATORPARC := 0;
      V_CONTA := 1;
      V_NRFATOR := 1;
      V_DTULTAUX := V_DTAUTOFORN;
      
      FOR V_CONTA IN 1..V_NRPARCPGTOAF LOOP

         SELECT
            LAST_DAY(V_DTAUTOFORN),LAST_DAY(V_DTAUTOFORN)-V_DTAUTOFORN
            INTO V_DTULDIAMES, V_NRDIAS
         FROM PARAMGERAL;

         V_DTULTAUX := V_DTAUTOFORN;
         WHILE V_DTPARCELA > V_DTULDIAMES LOOP

           SELECT MAX(MOVINDICE.VRINDICE) INTO V_VRINDICE
           FROM
            PARASUPRI, MOVINDICE
           WHERE
            PARASUPRI.CDFILIAL   = P_CDFILIAL           AND
            TO_CHAR(MOVINDICE.DTREFIND,'MMYYYY') =
            TO_CHAR(V_DTULDIAMES,'MMYYYY')               AND
            PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
            PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
            PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

           IF V_VRINDICE IS NULL THEN
              V_VRINDICE := 0;
           END IF;

           V_NRFATOR := V_NRFATOR * (1 + (V_VRINDICE/100))**(V_NRDIAS/30);

           V_DTULTAUX := V_DTULDIAMES;

           SELECT LAST_DAY(V_DTULDIAMES+1),LAST_DAY(V_DTULDIAMES+1)-V_DTULDIAMES
              INTO V_DTULDIAMES, V_NRDIAS
           FROM PARAMGERAL;

         END LOOP;

         V_NRDIAS := V_DTPARCELA - V_DTULTAUX;

         SELECT MAX(MOVINDICE.VRINDICE) INTO V_VRINDICE
         FROM
          PARASUPRI, MOVINDICE
         WHERE
          PARASUPRI.CDFILIAL   = P_CDFILIAL           AND
          TO_CHAR(MOVINDICE.DTREFIND,'MMYYYY') =
          TO_CHAR(V_DTPARCELA,'MMYYYY')               AND
          PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
          PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
          PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

         IF V_VRINDICE IS NULL THEN
            V_VRINDICE := 0;
         END IF;

         V_NRFATOR := V_NRFATOR * (1 + (V_VRINDICE/100))**(V_NRDIAS/30);

         V_FATORPARC := V_FATORPARC + (1/V_NRFATOR);

         SELECT ADD_MONTHS(V_DTPARCELA,1) INTO V_DTPARCELA
         FROM PARAMGERAL;

      END LOOP;

      V_VRPRECECO := ROUND(((V_VRPREUNIAF -
                     (V_VRPREUNIAF * (V_VRPERDESAF /100)) +
                      V_VRIMPITM) / V_NRPARCPGTOAF) *
                      (1/(1+(V_VRRENDATUMAR/100))) *
                      V_FATORPARC,5);

  ELSE
      V_VRPRECECO := ROUND((V_VRPREUNIAF -
                     (V_VRPREUNIAF * (V_VRPERDESAF /100)) +
                      V_VRIMPITM) *
                      (1/(1+(V_VRRENDATUMAR/100))),5);
  END IF;
  P_VRPRECECO := V_VRPRECECO;
END;
/