
// FILE: src/FloatingReportCardController.js
function FloatingReportCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {

	//------ Control variables
	var editingWidget = $scope.$parent.widget;

    /**
     * Checks if the card is already opened and reconfigure the state of it if it not.
     *
     * @param  {jquery.Event} e   Event that triggered the click.
     */
	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			}
			else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}
			}
		}
	}

	/**
	 * Toggle the report action when it's template is configured as card.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 */
	$scope.toggleFloatReportCard = function(e) {
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};

	editingWidget.toggleFloatReportCard = $scope.toggleFloatReportCard;

	/**
	 * Change the report orientation.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 * @param  {String} orientation   Report orientation.
	 */
	$scope.setOrientation = function(e, orientation) {
		$scope.widget.reportOrientation = orientation;
		e.stopPropagation();
	};
}

// FILE: src/ZhReports.js
/* global Util, Configuration*/
function ZhReports(templateManager, ZHPromise, ScreenService, restEngine, ZhUtil, eventAggregator, $filter, DecodeValueFieldService, ZhLogger, eventEngine) {

    var reportConfig = {
        PDF:{
            uri: '/generateReport',
            type: 'PDF',
            action: 'pdfAction'
        },
        XLS:{
            uri: '/generateXLS',
            type: 'XLS',
            action: 'xlsAction'
        },
        CSV:{
            uri: '/generateCSV',
            type: 'CSV',
            action: 'csvAction'
        }
    };

    var maxVisibleColumns = {
        'LANDSCAPE': 15,
        'PORTRAIT': 10
    };

    var reportNameToReportFunction = {
        "=": "is equal to",
        "IN": "is among",
        "NOT_IN": "is not among",
        "!=": "is different of",
        ">": "is bigger than",
        "<": "is lower than",
        ">=": "is not lower than",
        "<=": "is not bigger than",
        "BETWEEN": ["is between", "and"],
        "NOT_BETWEEN": ["is lower than", "and bigger than"],
        "LIKE": "has",
        "LIKE_I": "has",
        "LIKE_ALL": "has any of",
        "NOT_LIKE": "does not have",
        "IS_NULL": "is not defined",
        "IS_NOT_NULL": "is defined",
        "ORDER_DESC": "ordered descending",
        "ORDER_ASC": "ordered ascending",
        "ORDER_BY": "ordered",
        "GROUP_BY": "grouped"
    };

    eventEngine.registerEvent('BeforeGenerateReport', 'beforeGenerateReport');

    function triggerBeforeGenerateReport(widget, metaData){
        if(widget.beforeGenerateReport){
            return widget.beforeGenerateReport({
                'metadata': metaData
            });
        }
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Field}  field  - Field with the data related to the filter.
     * @param {Object} filter - Filter used to create the report.
     *
     * @return {String} Text to be shown woth data related to the given filter.
     */
    function getFieldFilterValue(field, filter) {
        return getFieldFilterDefaultText(filter, filter.value, ScreenService.i18n(reportNameToReportFunction[filter.operator]));
    }

    /**
     * Configure the default value for the variable if it's not defined.
     *
     * @param {Mixed} value        - The variable to be tested.
     * @param {Mixed} defaultValue - The value attributed the variable if it's not defined.
     *
     * @return {Mixed} The value that the variable should recieve.
     */
    function notUndefinedOrDefault(value, defaultValue) {
        return value === undefined ? defaultValue : value;
    }

    /**
     * Tests if the given type is PDF.
     *
     * @param {String} type - The type to be tested.
     *
     * @return {Boolean} Tells if the given type is PDF.
     */
    function isPdfFormat(type) {
        return type === reportConfig.PDF.type;
    }

     /**
     * Check which type of template need to be returned.
     *
     * @return {String} Template type to be used.
     */
    function checkZhReportActionDisplayMode() {
        if( !templateManager.project.zhReportActionDisplayMode ) {
            return 'zh-reports#floating-report-card.html';
        } else if (templateManager.project.zhReportActionDisplayMode === 'buttons') {
            return 'zh-reports#floating-report.html';
        }
    }

    eventAggregator.subscribe('beforeProjectStartup', function() {
        templateManager.project.customFloatingActions.push({
            'id': '#reportAction#',
            'name': 'reportAction',
            'label': ScreenService.i18n('Export'),
            "expandable": !templateManager.project.zhReportActionDisplayMode,
            'order': 10,
            'icon': 'file-export',
            'template': checkZhReportActionDisplayMode(),
            'isVisible': function(widget) {
                var show = true;
                if (widget.floatingControl) {
                    var showPdfAction = notUndefinedOrDefault(widget.floatingControl.pdfAction, true);
                    var showXlsAction = notUndefinedOrDefault(widget.floatingControl.xlsAction, true);
                    var showCsvAction = notUndefinedOrDefault(widget.floatingControl.csvAction, true);
                    show = showPdfAction || showXlsAction || showCsvAction;
                }

                return show;
            }
        });
    });

    /**
     * Checks if the propertie 'reportsUrl' is configured on the environment.json.
     *
     * @throws {Error} Message warning that the propertie should be configured at the file.
     */
    var checkConfiguration = function() {
        if (!(templateManager.project && templateManager.project.reportsUrl)) {
            throw (ScreenService.i18n(
                'Need report configuration in "environment.json": "reportsUrl"!'));
        }
    };

    /**
     * Returns the format of the column.
     *
     * @param {Field} column - The Field to test it's format.
     *
     * @return {Object} The Field's format.
     */
    var getFormatOfColumn = function(column) {
        var format = column.format;
        if (!format && column.mask) {
            column.mask.params = column.mask.params || {};
            format = {};
            format.params = column.mask.params;
            format.type = column.mask.type;
            if (Util.isString(format.params.mask) && Util.stringStartsWith(format.params.mask, '@') && Util.stringEndsWith(format.params.mask, '@')) {
                format.params.mask = ZhUtil.evaluate(format.params.mask);
            }
        }
        return format;
    };

    /**
     * Build a column to the metadata based on the given field.
     *
     * @param {Field}  column - Field base to create the column to the report.
     * @param {String} size   - The size of the Field, in percentage inside the report.
     *
     * @return {Object} Column to the metadata, with the necessary data.
     */
    var buildColumn = function(column, size) {
        if (Util.isUndefined(column.reportAlign)) {
            column.reportAlign = column.align || 'left';
        }
        return {
            align: column.reportAlign,
            description: ScreenService.i18n(column.labelOnReport || column.label),
            expression: column.expression,
            format: getFormatOfColumn(column),
            sequence: column.sequenceInList,
            size: size,
            xlsType: column.xlsType
        };
    };

    /**
     * Filter the widgetColumns by just the visible fields.
     *
     * @param {Array.<Field>}  widgetColumns               - Array to be filtered.
     * @param {Boolean}        reportShowOnlyVisibleFields - Tells if the report should show only visible fields.
     * @param {String}         [reportType='PDF']          - The type of the report.
     * @param {Array.<String>} [groups]                    - List of the field names that are grouped.
     *
     * @return {Array.<Field>} Filtered array with all fields that are visible on the report.
     */
    var filterVisibleColumns = function(widgetColumns, reportShowOnlyVisibleFields, reportType, groups) {
        var showGroupedColumns = !isPdfFormat(reportType);

        return widgetColumns.filter(function(column) {
            var isGroupedColumn = groups ? groups.indexOf(column.name) != -1 : column.grouped;
            var columnGroupVisibilitity = showGroupedColumns && isGroupedColumn;

            var isVisibleOnReport = Util.isUndefinedOrNull(column.isVisibleOnReport) || column.isVisibleOnReport;
            var columnVisibility = isVisibleOnReport && (!reportShowOnlyVisibleFields || (column.isVisible && column.showOnList));

            return column.aclIsVisible && (columnGroupVisibilitity || columnVisibility);
        });
    };

    /**
     * Returns the max visible columns on the report.
     *
     * @param  {Widget}    widget       -   Widget where the report is being generated.
     * @param  {String}    reportType   -   The type of the report.
     *
     * @return {Integer} The max of visible columns on the report.
     */
    var getMaxVisibleColumns = function(widget, reportType) {
        var maxColumns;
        if (reportType==reportConfig.PDF.type) {
            maxColumns = widget.maxColumns;
        } else {
            maxColumns = 99999;
        }
        return maxColumns;
    };

    /**
     * Builds the configuration of the field's display.
     *
     * @param {Widget}         widget             - Widget where the report is being generated.
     * @param {Array.<String>} [columnsToUse]     - The name of the fields that will be used on the
     * report generation.
     * @param {String}         [reportType='PDF'] - The type of the report.
     * @param {Array.<String>} [groups]           - List of the field names that are grouped.
     *
     * @return {Object} Object where each propertie is a object with the configuration of its display
     * on the report.
     */
    var _buildMetaDataColumns = function(widget, columnsToUse, reportType, groups) {
        var columns = {};
        var widgetColumns = widget.fields;
        var maxVisibleColCount = getMaxVisibleColumns(widget, reportType);
        if (widgetColumns) {
            widgetColumns = filterVisibleColumns(widgetColumns, widget.reportShowOnlyVisibleFields, reportType, groups);
            if (maxVisibleColCount && widgetColumns.length > maxVisibleColCount) {
                ZhLogger.warning(ScreenService.i18n(
                    "Number of columns too big to render the report correctly. It was limited to __maxVisibleColCount__ columns.",
                    {
                        'maxVisibleColCount': maxVisibleColCount
                    }
                ));
            }

            var remainingSize = 100;
            var columnsWithoutColumnSize = 0;

            widgetColumns.forEach(function(column) {
                if (column.reportColumnSize) {
                    remainingSize -= parseInt(column.reportColumnSize);
                }
                else {
                    columnsWithoutColumnSize++;
                }
            });

            if (remainingSize < 0) {
                ZhLogger.warning(ScreenService.i18n('Sum of columns width is bigger than 100%.'));
            }

            var size = Math.max(2, Math.floor(remainingSize / columnsWithoutColumnSize));
            var sum = 0;
            widgetColumns.forEach(function(column, index) {
                if (!columnsToUse || !columnsToUse[column.name]) {
                    var columnSize;
                    if (column.reportColumnSize) {
                        columnSize = column.reportColumnSize;
                    }
                    else {
                        if (index === (widgetColumns.length - 1)) {
                            columnSize = Math.max(2, remainingSize - sum);
                        }
                        else {
                            columnSize = size;
                            sum += size;
                        }
                    }

                    if (!columnSizeHasUnity(columnSize)) {
                        columnSize += '%';
                    }

                    columns[column.valueFieldOnReport || column.name] = buildColumn(column, columnSize);
                }
            });

        }
        return columns;
    };

    /**
     * Returns an array containing only the displayed value from field.
     *
     * @param {Object} object - Object containing the dataSource.data's row.
     *
     * @return {Object} Array containing only a object as {field.descriptionField: <displayedValue>}.
     */
    function getStaticPropertyValueFromObject(row) {
        var toReturn = {};
        toReturn[row[this.valueField]] = row[this.descriptionField];
        return toReturn;
    }

    /**
     * Builds the configuration of the field's staticData.
     *
     * @param {Widget} widget - Widget where the report is being generated.
     * @param {Object} object - Object were the configuration field's display are gathered.
     *
     * @return {Object} Object where all the staticData from fields with DataSource are
     * gathered to be treated.
     */
    var _buildStaticDataColumns = function(widget, columns) {
        var widgetColumns = widget.fields;
        var staticData = {};
        if (widgetColumns) {
            widgetColumns.forEach(function(column) {
                if (columns[column.name] && column.dataSource && column.dataSource.data) {
                    staticData[column.name] = column.dataSource.data.map(getStaticPropertyValueFromObject, column);
                }
                else if (columns[column.valueFieldOnReport] && column.dataSource && column.dataSource.data) {
                    staticData[column.valueFieldOnReport] = column.dataSource.data.map(getStaticPropertyValueFromObject, column);
                }
            });

            if (Object.keys(staticData).length) {
                for (var data in staticData) {
                    staticData[data] = staticData[data].reduce(function(o1, o2) {
                        for (var key in o2) {
                            o1[key] = o2[key];
                        }
                        return o1;
                    }, {});
                }
            }
        }

        return staticData;
    };

    /**
     * Tests if the column size has a measurement unit.
     *
     * @param {String|Number} columnSize - The column size to be tested.
     *
     * @return {Boolean} Tells if the column size has a measurement unit.
     */
    function columnSizeHasUnity(columnSize) {
        return (/[\%px]/g).test(columnSize.toString());
    }

    /**
     * Build a report's group based on the given Field.
     *
     * @param {Field} column - The Field to be based on the cration of the column.
     *
     * @return {Object} Report's group.
     */
    var buildGroup = function(column) {
        return {
            name: column.name,
            field: column.name,
            isNumber: column.isNumber,
            descending: column.descending,
            format: getFormatOfColumn(column),
            internalSorts: column.internalSorts,
            header: {
                label: column.headerLabel || ScreenService.i18n(column.label),
                displayfield: column.name
            }
        };
    };

    /**
     * Build the metadata of the fields grouped.
     *
     * @param {Widget} widget - Widget used to create the grouped metadata.
     * @param {Array.<String>} [customGroups] - List of the field names that are grouped.
     *
     * @return {Array.<Object>} Metadata of grouped columns.
     */
    var _buildMetaDataGroups = function(widget, customGroups) {
        var buildedGroups = [];
        var groups = widget.groups;
        if (customGroups) {
            groups = customGroups.map(function(name) {
                return widget.getField(name);
            });
        }
        buildedGroups = groups.map(function(group) {
            return buildGroup(group);
        });
        return buildedGroups;
    };

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function getFieldFilterDefaultTextWithoutValue(text) {
        return ScreenService.i18n(text) + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function likeIFieldText(value, text) {
        return ScreenService.i18n(text) + ' \'' + value.replace(/%/g, '') + '\' case sensitive.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     * @param {Object} filter - Filter used to create the report.
     *
     * @return {String} Text to be shown in the header.
     */
    function valueFieldText(filter, value, text) {
        if (value[0] === "%" && value[value.length - 1] === "%") {
            value = value.substring(1, value.length - 1);
        }
        else {

            if (value[0] === "%") {
                if (filter.operator == 'LIKE') {
                    value = value.substring(1, value.length);
                    text = "ends with";
                }
                else if (filter.operator == 'NOT_LIKE') {
                    value = value.substring(1, value.length);
                    text = "does not ends with";
                }
            }
            if (value[value.length - 1] === "%") {
                if (filter.operator == 'LIKE') {
                    value = value.substring(0, (value.length - 1));
                    text = "starts with";
                }
                else if (filter.operator == 'NOT_LIKE') {
                    value = value.substring(0, (value.length - 1));
                    text = "does not starts with";
                }
            }
        }
        return ScreenService.i18n(text) + ' ' + value + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function oneOrMoreValuesFieldText(value, text) {
        if (Array.isArray(text)) {
            return ScreenService.i18n(text[0]) + ' ' + value[0] + ' ' + ScreenService.i18n(text[1]) + ' ' + value[1] + '.';
        }

        return ScreenService.i18n(text) + ' ' + value.join(', ') + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} filter- Filter used to create the report.
     * @param {Object} value - Value of the filter.
     * @param {String} text  - Text used to create the header of the report.
     *
     * @return {String} Text to be shown woth data related to the given filter.
     */
    var getFieldFilterDefaultText = function(filter, value, text) {
        if (Array.isArray(value)) {

            return oneOrMoreValuesFieldText(value, text);
        }

        if (filter.operator === "LIKE_I") {
            return likeIFieldText(value, text);
        }

        if (!value) {
            return getFieldFilterDefaultTextWithoutValue(text);
        }

        return valueFieldText(filter, value, text);


    };

    /**
     * Checks if the filter related to the given item should be displayed on the header of
     * the report.
     *
     * @param  {Field} field -  Field to be filter to be tested.
     *
     * @return {Boolean} Boolean telling if the filter related to the field should be displayed.
     */
    function filterIsVisibleOnHeader(field) {
        return field && field.isVisibleOnReport;
    }

    /**
     * Builds the metadata of the filter.
     *
     * @param {Widget} widget - Widget where the report is being generated.
     *
     * @return {Array.<Object>} The metadatas of each filter in the widget.
     */
    var _buildMetaDataFilter = function(widget) {
        var filterData = [];
        if (widget.dataSourceFilter) {
            widget.dataSourceFilter.forEach(function(item) {
                var field = widget.getField(item.name);
                if (filterIsVisibleOnHeader(field)) {
                    filterData.push({
                        label: field.label,
                        value: ZhUtil.evaluate(getFieldFilterValue(field, item))
                    });
                }
            });
        }

        return filterData;
    };

    /**
     * Retrieves the image from the url and converts it to base64
     *
     * @param {String}   url    - The source of the image
     * @param {Function} widget - Widget where the report is being generated.
     */
    function convertImageToB64(url, callback) {
        var xhr = new XMLHttpRequest();

        xhr.onload = function() {
            var reader = new FileReader();
            reader.onloadend = function() {
                callback(reader.result);
            };
            reader.readAsDataURL(xhr.response);
        };

        xhr.open('GET', url);
        xhr.responseType = 'blob';
        xhr.send();
    }

    /**
     * Fetches the images if they exist.
     *
     * @param {Function} callback  - The callback to be executed after all the images where fetched.
     * @param {Widget}   widget    - Widget where the report is being generated.
     */
    function getAllImages(container, callback) {
        var imgs = {};
        var imgSrc = ['reportClientLogo', 'reportProductLogo', 'reportEnterpriseLogo'];
        var index = 0;

        if(!container[imgSrc[0]] && !container[imgSrc[1]] && !container[imgSrc[2]]){
            callback({});
            return;
        }

        imgSrc.forEach(function(imgName){
            if(container[imgName]){
                index++;
                convertImageToB64(container[imgName], function(imageData){
                    imgs[imgName] = imageData;
                    index--;

                    if(index === 0){
                        callback(imgs);
                    }
                });
            }
        });
    }

    /**
     * Builds the widget metadata.
     *
     * @param {Object} [customData]       - Customization data for the generation of the report.
     * @param {Widget} sourceWidget       - Widget where the report is being generated.
     * @param {String} [reportType='PDF'] - The type of the report.
     *
     * @return {Object} The widget's metadata for the generation of the report.
     */
    var buildMetaData = function(customData, sourceWidget, reportType, callback) {
        var widget = sourceWidget;
        var columns = _buildMetaDataColumns(widget, customData.columns, reportType, customData.groups);
        var staticColumns = _buildStaticDataColumns(widget, columns);
        var widgetLabel = ScreenService.i18n(widget.label);

        getAllImages(templateManager.project, function(images) {
            var widgetMetaData = {
                name: widget.name,
                title: customData.name || widgetLabel,
                reportName: customData.reportName || widget.name,
                reportClientLogo: images.reportClientLogo,
                reportProductLogo: images.reportProductLogo,
                reportEnterpriseLogo: images.reportEnterpriseLogo,
                widgetLabel: widgetLabel,
                version: '1.0',
                clientLogo: templateManager.project.reportClientLogo,
                productLogo: templateManager.project.reportProductLogo,
                enterpriseLogo: templateManager.project.reportEnterpriseLogo,
                columns: columns,
                staticData: staticColumns,
                groups: _buildMetaDataGroups(widget, customData.groups),
                filter: _buildMetaDataFilter(widget),
                xlsDefaultType: widget.xlsDefaultType,
                orientation: widget.reportOrientation || 'PORTRAIT'
            };

            callback(widgetMetaData);
        });
    };


    /**
     * Tells if the given field needs to have it's value decoded.
     *
     * @param {Field} field - Fild to test if it's value need to be decoded.
     *
     * @return {Boolean} Tells if the field's value need to be decoded
     */
    var shouldUseDecodeValueField = function(field) {
        var masksTypesToAvoid = ['currency', 'hour', 'float', 'percent', 'fix'];
        return !field.mask || masksTypesToAvoid.indexOf(field.mask.type) == -1;
    };

    /**
     * Get the grouped columns from the given metadata of grouped fields.
     *
     * @param {Object} groupsMetadata - Metadata of the grouped on the report.
     *
     * @return {Array.<Field>} The fields of the group.
     */
    var getGroupedColumns = function(groupsMetadata) {
        var columns = [];
        groupsMetadata.forEach(function(group) {
            columns.push(group.field);
        });

        return columns;
    };

    /**
     * Format the values of the dataset based on the field of the widget configured
     * the metaData.
     *
     * @param {Array.<Object>} dataSet  - The widget's dataset.
     * @param {Object}         metaData - Metadata of the report.
     * @param {Widget}         widget   - Widget where the report is being generated.
     *
     * @return {Array.<Object>} The given dataset formatted for be used on the report.
     */
    var formatValuesForReport = function(dataSet, metaData, widget) {
        var columns = Object.keys(metaData.columns);
        columns = columns.concat(getGroupedColumns(metaData.groups));

        return dataSet.map(function(row) {
            if(row.__groupHeader || row.__groupFooter) return row;

            var formattedRow = {};
            columns.forEach(function(column) {
                var field = widget.getField(column);
                if (field && shouldUseDecodeValueField(field)) {
                    formattedRow[column] = DecodeValueFieldService.model(field, row[column]);
                }
                else {
                    formattedRow[column] = row[column];
                }
            });

            return formattedRow;
        });
    };

    /**
     * Builds the dataSet based on the given widget,
     *
     * @param {Object} metaData   - The metadata of the report.
     * @param {Widget} widget     - Widget where the report is being generated.
     * @param {String} reportType - The report type to generate.
     *
     * @return {Array.<Object>}  Dataset from the widget formatted to the report.
     */
    var buildDataSet = function(metaData, widget, reportType) {
        return loadAllData(widget, reportType).then(function(dataSet) {
            return isPdfFormat(reportType) ? formatValuesForReport(dataSet, metaData, widget) : dataSet;
        });
    };

    /**
     * Get the datasorce's name without the initial '/'.
     *
     * @param {DataSource} dataSource - Widget's dataSource to get the name.
     *
     * @return {String} Datasorce name without the initial '/'.
     */
    function getDataSourceName(dataSource) {
        return dataSource.name.split('/').pop();
    }

    /**
     * Load all data from the widget's dataSource.
     *
     * @param {Widget} widget     - Widget where the report is being generated.
     * @param {String} reportType - The report type to generate.
     *
     * @return {Promise.<Array>} Promise that will be resolved with the widget's datasource.
     */
    function loadAllData(widget, reportType) {
        var dataSource = widget.dataSource;
        var storageStrategy = dataSource.getStorageStrategy();
        var result;

        if (storageStrategy) {
            result = storageStrategy.getDataSource(dataSource.name, widget.getDataSourceFilter(), null).then(function(dataSet) {
                if (Util.isObject(dataSet)) {
                    dataToReturn = dataSet.dataset[getDataSourceName(dataSource)];
                }
                else {
                    dataToReturn = dataSet;
                }

                return isPdfFormat(reportType) ? widget.getUpdatedGroupData(dataToReturn) : dataToReturn;
            });
        }
        else {
            result = ZHPromise.when(widget.groupedData);
        }

        return result;
    }

    /**
     * Build the path where the file was created.
     *
     * @param {String} fileName - Name of the created file.
     *
     * @return {Object} Object with the ful path of the created file.
     */
    var buildFilePath = function(fileName) {
        var reportsUrl = templateManager.project.reportsUrl;
        return {
            path: reportsUrl + '/reports/' + fileName
        };
    };

    /**
     * Builds a successs callback.
     *
     * @param {Function} callback - Function to be executed with the FilePath.
     *
     * @return {Function} Callback with the given callback.
     */
    var buildCallbackOfSuccess = function(callback) {
        return function(data) {
            if (callback) {
                callback(buildFilePath(data.messages[0].message));
            }
        };
    };

    /**
     * Build the file path of the generated file for the dataExporter
     *
     * @param {String} fileName - Name of the generated file.
     *
     * @return {Object} Object with the fukk path of the generated file.
     */
    var buildFilePathForDataExporter = function(exporterUrl, fileName) {
        return {
            path: exporterUrl + fileName
        };
    };

    /**
     * Builds a successs callback for the dataExporter.
     *
     * @param {Function} callback - Function to be executed with the FilePath.
     *
     * @return {Function} Callback with the given callback.
     */
    var buildCallbackOfSuccessForDataExporter = function(exporterUrl, callback) {
        return function(data) {
            if (callback) {
                callback(buildFilePathForDataExporter(exporterUrl, data.messages[0].message));
            }
        };
    };

    /**
     * Returns the uri from a given report type.
     *
     * @param {String} reportType - Report type that will be used to get the uri.
     *
     * @return {String} - Uri from the given report type.
     */
    var getReportUri = function(reportType) {
        return reportConfig[reportType].uri;
    };

    /**
     * Format the dataset for the XLS and CSV files.
     *
     * @param {Object} columns - The metadata of each column of the report.
     * @param {Array}  dataSet - The datasource's data of the widget.
     *
     * @return {Array.<Object>} The dataSet to the XLS and CSV files
     */
    var formatDataSet = function(columns, dataSet) {
        var columnsName = Object.keys(columns);
        var rowValues = [];
        var formatedDataSet = [];

        dataSet.forEach(function(row) {
            columnsName.forEach(function(column) {
                rowValues.push(row[column]);
            });

            formatedDataSet.push(rowValues);
            rowValues = [];
        });

        return [columnsName].concat(formatedDataSet);
    };

    /**
     * Build the dataSet to be used on the backend to generate the report.
     *
     * @param {String} reportType - Defines what report type will be generate.
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} dataSet - Data used to create the reports.
     *
     * @return {Object} - Builded dataSet.
     */
    var buildDataSetToReportRow = function(reportType, metaData, dataSet) {
        if(reportType == reportConfig.PDF.type) {
            return dataSet.map(ZhUtil.clearRow);
        } else {
            return formatDataSet(metaData.columns, dataSet);
        }
    };

    /**
     * Build the reportRow to be used on the backend to generate the report.
     *
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} reportRowDataSet - Data used to create the reports.
     * @param {Widget} widget - Widget to report.
     *
     * @return {Object} reportRow - Builded reportRow.
     */
    var buildReportRow = function(metaData, reportRowDataSet, widget) {
        var reportRow = {};
        reportRow.metaData = JSON.stringify(metaData);
        reportRow.dataSet = JSON.stringify(reportRowDataSet);
        reportRow.filter = JSON.stringify(widget.dataSourceFilter);
        return reportRow;
    };

    /**
     * Build the params to be used on the request.
     *
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} reportRowDataSet - Data used to create the reports.
     * @param {Widget} widget - Widget to report.
     *
     * @return {Object} - Builded params.
     */
    var buildParamsToRequest = function(metaData, reportRowDataSet, widget) {
        return {
            'requestType': 'Row',
            'row': buildReportRow(metaData, reportRowDataSet, widget)
        };
    };

    /**
     * Build the request to generate the report.
     *
     * @param {String} reportType         - Defines what report type will be generate.
     * @param {Object} metaData           - Defines the report's structure.
     * @param {Object} reportRowDataSet   - Data used to create the reports.
     * @param {function} callback         - Callback called after sucess.
     * @param {function} callbackError    - Callback called after error.
     * @param {Widget} widget             - Widget to report.
     *
     * @return {Promisse} Request's promisse.
     */
    var buildRequest = function(reportType, metaData, reportRowDataSet, callback, callbackError, widget) {
        var uri = getReportUri(reportType);
        var params = buildParamsToRequest(metaData, reportRowDataSet, widget);
        var callbackSuccess = buildCallbackOfSuccess(callback);
        var requestType = 'post';
        var reportsUrl = templateManager.project.reportsUrl;

        return restEngine.request(uri,params,callbackSuccess,requestType,callbackError,reportsUrl);
    };

    /**
     * Process to be executed after the request.
     *
     * @param {Object}      response        - Request result.
     * @param {Promisse}    deferred        - Promisse that defines the result of report generation.
     * @param {String}      reportType      - Defines what report type will be generated.
     *
     */
    var afterRequest = function(widget, response, deferred, reportType) {
        if (response) {
            var file = buildFilePath(response.messages[0].message);

            deferred.resolve(file);

            if(widget.afterOpenreport){
                widget.afterOpenreport();
            }
        }
        else {
            deferred.reject();
            publishErrorEvent();
        }
    };

    /**
     * Generates the report making a requests to the server to generate the file.
     *
     * @param {Object}          metaData        - Metadata for the construction of the report.
     * @param {Array.<Object>}  dataSet         - The widget's dataset.
     * @param {Function}        callback        - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function}        callbackError   - Callback that will be executed if the request failed.
     * @param {String}          reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created of rejected if somethig went wrong.
     */
    var _generateReport = function(widget, metaData, dataSet, callback, callbackError, reportType) {
        var deferred = ZHPromise.defer();
        var reportRowDataSet = buildDataSetToReportRow(reportType, metaData, dataSet);

        buildRequest(reportType, metaData, reportRowDataSet, callback, callbackError, widget).then(function(response) {
            afterRequest(widget, response, deferred, reportType);
        }).catch(function(response) {
            deferred.reject(response);
            publishErrorEvent();
        });

        return deferred.promise;
    };

    /**
     * Returns the params to make the request.
     *
     * @param {Object} actionControl - The widget's actionControl.
     * @param {Widget} widget        - Widget where the report is being generated.
     * @param {Object} metaData      - Metadata for the construction of the report.
     *
     * @return {Object} Params to the request.
     */
    function getParams(actionControl, widget, metaData){
        var params = {
            'requestType': 'FilterData',
            'filter': widget.dataSourceFilter,
            'metaData': metaData
        };
        if(actionControl.dataSourceName){
            params.dataSourceName = actionControl.dataSourceName;
        } else if(actionControl.collectionName){
            params.collectionName = actionControl.collectionName;
        } else {
            var reportRow = {};
            reportRow.metaData = JSON.stringify(metaData);
            reportRow.dataSet = JSON.stringify(widget.dataSource.data);
            reportRow.filter = JSON.stringify(widget.dataSourceFilter);

            params = {
                'requestType': 'Row',
                'row': reportRow,
            };
        }
        return params;
    }

    /**
     * Generates the report making only one request to the server generate the file.
     *
     * @param {Object}   metaData           - Metadata for the construction of the report.
     * @param {Object}   actionControl      - The widget's actionControl.
     * @param {Object}   widget             - Widget where the report is being generated.
     * @param {Function} callback           - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function} callbackError      - Callback that will be executed if the request failed.
     * @param {String}   [reportType='PDF'] - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created of rejected if somethig went wrong.
     */
    var _generateServerSideReport = function(metaData, actionControl, widget, callback, callbackError, reportType) {
        var exporterUrl = templateManager.project.exporterUrl;
        var deferred = ZHPromise.defer();
        var uri = getReportUri(reportType);
        var params = getParams(actionControl, widget, metaData);

        restEngine.request(uri, params, buildCallbackOfSuccessForDataExporter(exporterUrl, callback), 'post', callbackError, exporterUrl)
            .then(
                function(response) {
                    afterRequest(widget, response, deferred, reportType);
                })
            .catch(
                function(response) {
                    deferred.reject(response);
                    publishErrorEvent();
                }
            );

        return deferred.promise;
    };

    /**
     * Publish a new event. Lauched after the report failed.
     */
    var publishErrorEvent = function() {
        eventAggregator.publish('afterReportError');
    };

    /**
     * Creates a metaData for the translate of fixed words on the report.
     *
     * @param {Object} metaData - Metada of the report.
     *
     * @return {Object} The metadata with the configuration of the i18n
     */
    var _i18nMetaData = function(metaData) {
        var words = {
            'Page': ScreenService.i18n('Page'),
            'Report': ScreenService.i18n('Report'),
            'Version': ScreenService.i18n('Version')
        };

        metaData.words = words;
        metaData.lang = templateManager.lang;

        Object.keys(metaData.columns).forEach(function(key) {
            var column = metaData.columns[key];

            column.description = ScreenService.i18n(column.description);
        });

        metaData.groups.forEach(function(group) {
            Object.keys(group).forEach(function(key) {
                var grp = group[key];
                if (grp) {
                    grp.label = ScreenService.i18n(grp.label);
                }
            });
        });

        return metaData;
    };

    /**
     * Get the action control's configuration.
     *
     * @param  {Widget} widget     - Widget where the report is being generated.
     * @param  {String} reportType - The type of the report.
     *
     * @return {Object | Boolean} A object containing the configuration of the floating action or true.
     */
    var getActionControl = function(widget, reportType) {
        return Util.getNestedProperty(widget, ['floatingControl', reportConfig[reportType].action]);
    };

    /**
     * Generates the report.
     *
     * @param {Object}      customData      - Informations to customize the report.
     * @param {Widget}      sourceWidget    - Widget where the report is being generated.
     * @param {Function}    callback        - Function to be executed after the file be generated, reciving the name of the generated file.
     * @param {Function}    callbackError   - Function to be executed if happen a error while generating the file.
     * @param {String}      reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created o rejected.
     */
    this.generateReport = function generateReport(customData, sourceWidget, callback, callbackError, reportType) {
        if (Util.getNestedProperty(sourceWidget, ['dataSource', 'data', 'length'])) {
            customData = customData || {};
            checkConfiguration();
            var widget = sourceWidget;
            var promise;

            buildMetaData(customData, widget, reportType, function(metaDataResolved) {
                var metaDataInternationalized = _i18nMetaData(metaDataResolved);

                triggerBeforeGenerateReport(widget, metaDataInternationalized);

                var actionControl = getActionControl(widget, reportType);

                if (typeof actionControl === "object" && actionControl.runAt == 'server') {
                    promise = _generateServerSideReport(metaDataInternationalized, actionControl, widget, callback, callbackError, reportType);
                }
                else {
                    promise = buildDataSet(metaDataInternationalized, widget, reportType).then(function(dataSet) {
                        return _generateReport(widget, metaDataInternationalized, dataSet, callback, callbackError, reportType);
                    });
                }
            });

            return promise;
        }
        else {
            ScreenService.errorNotification("Can't create a report without data!");
        }
    };

    /**
     * Returns a callback to be executed if the report is created succefully.
     *
     * @param {Function} callback - Function executed after the test.
     *
     * @return {Function} Callback that tests if the screen is able to open the popup creating a fake popup and, warning the user if the windows's popups are blocked.
     */
    var _previewCallback = function(callback) {
        return function(data) {
            var popup_window = window.open(data.path, '_blank');
            if (!popup_window) {
                ScreenService.showMessage(ScreenService.i18n('Pop-up Blocker is enabled! Please add this site to your exception list.'));
            }
            if (callback) {
                callback(data);
            }
        };
    };

    /**
     * Creates the report and open it rigth after.
     *
     * @param {Object}      customData      - Custom data for the generation of the report.
     * @param {Widget}      sourceWidget    - Widget where the report is being generated.
     * @param {Function}    callback        - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function}    callbackError   - Callback that will be executed if the request failed.
     * @param {String}      reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created or rejected
     */
    this.openReport = function openReport(customData, sourceWidget, callback, callbackError, reportType) {
        return this.generateReport(customData, sourceWidget, _previewCallback(callback), callbackError, reportType);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhReports', ZhReports);
});


// FILE: src/ZhReportsDirective.js
var ZeedhiDirectives = angular.module('ZeedhiDirectives');

function ReportsDirective(ApplicationContext){
    return {
        priority: 2,
        link: function($scope) {
            /**
             * Generate the report of the given widget.
             * @param  {Widget}   widget  Widget where the report is being generated.
             * @param  {String}   type    The type of the report.
             */
            $scope.generateReport = function(widget, type) {
                ApplicationContext.ZhReports.openReport(widget.reportCustomData, widget, undefined, undefined, type);
            };
        }
    };
}

ZeedhiDirectives.directive(
    'zhFloatingControl', ['ApplicationContext', ReportsDirective]
);
